import os
import subprocess
import shutil

from charmhelpers.core.hookenv import log, status_set, charm_dir
from charms.reactive import (hook, when, when_not, when_any, set_flag,
                             set_state, remove_state)

APT_SOURCE_FNAME = "/etc/apt/sources.list.d/charm-autopkgtest.list"


def apt_update():
    subprocess.run(["apt-get", "-y", "update"], check=True)


def get_release_codename():
    with os.popen("lsb_release -cs") as fp:
        return fp.read().strip()


def add_apt_sources():
    """Add an APT source list with the required packages.

    This may duplicate some existing entries, but apt seems to handle
    this situation pretty well.
    """

    codename = get_release_codename()
    channels = "main universe restricted"
    with open(APT_SOURCE_FNAME, "w") as fp:
        fp.write("deb-src http://archive.ubuntu.com/ubuntu %s %s\n"
                 % (codename, channels))
        fp.write("deb-src http://security.ubuntu.com/ubuntu %s-security %s\n"
                 % (codename, channels))
        fp.write("deb-src http://archive.ubuntu.com/ubuntu %s-updates %s\n"
                 % (codename, channels))
    apt_update()


def remove_apt_sources():
    try:
        os.unlink(APT_SOURCE_FNAME)
    except FileNotFoundError:
        # Ignore non-existing files.
        pass
    apt_update()


def install_test_provider():
    fname = "autopkgtest-virt-null-reversible"
    src = os.path.join(charm_dir(), "files", fname)
    dst = os.path.join("/usr/local/bin/", fname)
    shutil.copyfile(src, dst)
    os.chmod(dst, 0o755)


@when_not('autopkgtest.ready')
def autopkgtest_ready():
    install_test_provider()
    try:
        add_apt_sources()
    except subprocess.CalledProcessError:
        status_set('blocked', "Failed to install dependencies")
    else:
        status_set('active', 'ready')
        set_flag('autopkgtest.ready')
